#!/usr/bin/env node

import { execSync } from "child_process";
import { readFileSync, existsSync, access } from "fs";
import firebase from "firebase/compat/app";
import "firebase/compat/firestore";
import "firebase/compat/auth";
import { intro, outro, spinner, text } from "@clack/prompts";
import capitalizedName from "./utils/getUserInfo.js";
import colorText from "./utils/textUtils.js";
import axios from "axios";

const s = spinner();

const firebaseConfig = {
  apiKey: "AIzaSyABm9M7XceFH6pSvvbMuRJw3n5nBeak3L0",
  authDomain: "share-brewfiles.firebaseapp.com",
  projectId: "share-brewfiles",
  storageBucket: "share-brewfiles.appspot.com",
  messagingSenderId: "1071283599196",
  appId: "1:1071283599196:web:a4910843f1bae76d7af399",
  measurementId: "G-WGD9RH2E7M",
};

firebase.initializeApp(firebaseConfig);

function isHomebrewInstalled() {
  try {
    execSync("brew --version");
    return true;
  } catch (error) {
    return false;
  }
}

async function authenticateWithGitHub() {
  try {
    await startClackSpinner(
      "Before uploading your Brewfile, we ask that you connect it with a Github account."
    );
    await stopClackSpinner(
      "Please login through the browser window that's about to open..."
    );
    execSync(
      "open https://github.com/login/oauth/authorize?client_id=6db8c19033a30994dffa&scope=read:user"
    );

    const code = await text({
      message: "Paste the code you were given here:",
      placeholder: "",
      initialValue: "",
    });

    return code;
  } catch (error) {
    return null;
  }
}

async function exchangeCodeForAccessToken(code) {
  try {
    const response = await axios.post(
      `https://www.brewfiles.com/api/exchangeCodeForAccessToken`,
      {
        code: code,
      },
      {
        headers: {
          Accept: "application/json",
        },
      }
    );
    const accessToken = response.data.access_token;
    await stopClackSpinner("Received your access token!");
    return accessToken;
  } catch (error) {
    return null;
  }
}

async function startClackSpinner(text) {
  s.start(text);
  await new Promise((resolve) => setTimeout(resolve, 2000));
}

async function stopClackSpinner(text) {
  s.stop(text);
  await new Promise((resolve) => setTimeout(resolve, 2000));
}

function installHomebrew() {
  execSync(
    '/bin/bash -c "$(curl -fsSL https://raw.githubusercontent.com/Homebrew/install/HEAD/install.sh)"'
  );
}

async function createNewBrewfile() {
  execSync("brew bundle dump");
}

async function getBrewfile() {
  await startClackSpinner("Checking for existing Brewfile");
  if (existsSync("Brewfile")) {
    await stopClackSpinner("Found existing Brewfile!");
  } else {
    await stopClackSpinner("No existing Brewfile found!");
    await startClackSpinner("Generating Brewfile for you");
    createNewBrewfile();
    await stopClackSpinner("Finished generating new Brewfile!");
  }
}

async function readBrewfileData() {
  const brewfileData = readFileSync("Brewfile", "utf-8");
  return brewfileData;
}

async function uploadBrewfileToFirebase(brewfileData, accessToken) {
  let id;
  await startClackSpinner("Uploading Brewfile to our website");
  const data = {
    brewfileData: brewfileData,
    accessToken: accessToken,
  };
  await axios
    .post("https://www.brewfiles.com/api/uploadBrewfile", data, {
      headers: {
        "Content-Type": "application/json",
      },
    })
    .then((response) => {
      id = response.data.id;
      stopClackSpinner("Successfully uploaded your Brewfile!");
    })
    .catch((error) => {
      stopClackSpinner(
        `We couldn't upload your Brewfile: ${error.response.data.message}`
      );
      throw new Error(error);
    });
  return id;
}

async function parseBrewfile(brewfileData) {
  const packages = [];
  const lines = brewfileData.split("\n");

  lines.forEach((line) => {
    if (line.startsWith("tap")) {
      const [, nameWithQuotes] = line.split(" ");
      const name = nameWithQuotes.replace(/^"(.*)"$/, "$1");
      packages.push({ name, packageManager: "tap" });
    } else if (line.startsWith("brew")) {
      const [, nameWithQuotes] = line.split(" ");
      const name = nameWithQuotes.replace(/^"(.*)"$/, "$1");
      packages.push({ name, packageManager: "brew" });
    } else if (line.startsWith("vscode")) {
      const [, nameWithQuotes] = line.split(" ");
      const name = nameWithQuotes.replace(/^"(.*)"$/, "$1");
      packages.push({ name, packageManager: "vscode" });
    }
  });
  return packages;
}

async function main() {
  let id;
  try {
    console.clear();
    console.log("📦 📦 📦 📦 📦 📦 📦 📦 📦 📦 📦 📦 📦 📦");
    console.log();
    intro(
      `Welcome to ${colorText(
        "Share Brewfiles",
        "BLACK",
        "GREEN"
      )}, ${capitalizedName}!`
    );
    if (!isHomebrewInstalled()) {
      installHomebrew();
    } else {
      await getBrewfile();
      const brewfileData = await readBrewfileData();
      if (brewfileData) {
        const packages = await parseBrewfile(brewfileData);
        const code = await authenticateWithGitHub();
        const accessToken = await exchangeCodeForAccessToken(code);
        id = await uploadBrewfileToFirebase(packages, accessToken);
      }
    }
    if (id) {
      outro(
        `${colorText(
          "Done! Now check out your brewfile here:",
          "BLACK",
          "GREEN"
        )}`
      );
      console.log(`👉 https://www.brewfiles.com/brew/${id} 👈`);
    } else {
      outro(
        `${colorText(
          "Done! Now check out the community leaderboard here:",
          "BLACK",
          "GREEN"
        )}`
      );
      console.log(`👉 https://www.brewfiles.com 👈`);
    }
    console.log();
    console.log("📦 📦 📦 📦 📦 📦 📦 📦 📦 📦 📦 📦 📦 📦");
    process.exit(0);
  } catch (error) {
    console.log();
    console.log("📦 📦 📦 📦 📦 📦 📦 📦 📦 📦 📦 📦 📦 📦");
    console.log();
    console.log(
      `Something went wrong! Try uploading your Brewfile again. ${colorText(
        "Exiting the program.",
        "WHITE",
        "MAGENTA"
      )}`
    );
    console.log();
    console.log("📦 📦 📦 📦 📦 📦 📦 📦 📦 📦 📦 📦 📦 📦");
    process.exit(0);
  }
}

main();
